<?php

namespace addons\xhpay\controller;

use addons\xhpay\library\Service;
use addons\xhpay\library\QRCode;
use think\addons\Controller;
use think\Response;
use think\Session;

/**
 * API接口控制器
 *
 * @package addons\xhpay\controller
 */
class Api extends Controller
{

    protected $layout = 'default';
    protected $config = [];

    /**
     * 默认方法
     */
    public function index()
    {
        $this->error();
    }

    /**
     * 外部提交
     */
    public function submit()
    {
        $out_trade_no = $this->request->request("out_trade_no");
        $body = $this->request->request("body");
        $total_fee = $this->request->request('total_fee');
        $type = $this->request->request('type');
        $method = $this->request->request('method', 'web');
        $notify_url = $this->request->request('notify_url', '');
        $return_url = $this->request->request('return_url', '');

        if (!$total_fee || $total_fee < 0) {
            $this->error("支付金额必须大于0");
        }

        if (!$type || !in_array($type, ['alipay', 'wechat'])) {
            $this->error("支付类型错误");
        }

        $params = [
            'type'         => $type,
            'out_trade_no' => $out_trade_no,
            'body'        => $body,
            'total_fee'       => $total_fee,
            'method'       => $method,
            'notify_url'    => $notify_url,
            'return_url'    => $return_url,
        ];
        return Service::submitOrder($params);
    }

    /**
     * 微信支付
     * @return string
     */
    public function wechat()
    {
        $config = Service::getConfig();
        
        //发起PC支付(Native支付)
        $params = Session::get("wechatresponse");
        
        $data = [
            'body'         => $params['body'],
            'code_url'     => $params['code_url'],
            'out_trade_no' => $params['out_trade_no'],
            'return_url'   => $params['return_url'],
            'total_fee'    => $params['total_fee'],
            'order_id'    => $params['order_id']
        ];
        if ($this->request->isAjax()) {
            $result = Service::query(['order_id'=>$params['order_id']]);
            if ($result['return_code'] == 'SUCCESS') {
               $this->success("", "", ['status' => $result['status']]);
            } else {
                $this->error("查询失败");
            }
        }
        $type = 'pc';
        $this->view->assign("data", $data);

        $this->view->assign("type", $type);
        $this->view->assign("title", "微信支付");
        return $this->view->fetch();
    }
    
    public function alipay(){
        $config = Service::getConfig();
        
        $params = Session::get("alipayresponse");

        $data = [
            'body'         => $params['body'],
            'code_url'     => $params['code_url'],
            'out_trade_no' => $params['out_trade_no'],
            'return_url'   => $params['return_url'],
            'total_fee'    => $params['total_fee'],
            'order_id'    => $params['order_id']
        ];
        if ($this->request->isAjax()) {
            $result = Service::query(['order_id'=>$params['order_id']]);
            if ($result['return_code'] == 'SUCCESS') {
               $this->success("", "", ['status' => $result['status']]);
            } else {
                $this->error("查询失败");
            }
        }
        $this->view->assign("data", $data);
        $this->view->assign("title", "支付宝支付");
        return $this->view->fetch();
    }

    /**
     * 生成二维码
     * @return Response
     */
    public function qrcode()
    {
        $text = $this->request->get('text', 'xhpay');
        $qr = QRCode::getMinimumQRCode($text);
        $im = $qr->createImage(8, 5);
        header("Content-type: image/png");
        imagepng($im);
        imagedestroy($im);
        return;
    }

}
