<?php

namespace addons\xhpay\library;

use Exception;
use think\Log;
use think\Response;
use think\Session;

/**
 * 订单服务类
 *
 * @package addons\xhpay\library
 */
class Service
{
    public static function submitOrder($amount, $out_trade_no = null, $type = null, $body = null, $notify_url = null, $return_url = null, $method = null, $attach=null, $goods_detail=null)
    {
        if (!is_array($amount)) {
            $params = [
                'total_fee'    => $amount,
                'out_trade_no'   => $out_trade_no,
                'body'     => $body,
                'notify_url' => $notify_url,
                'type'      => $type,
                'goods_detail'      => $goods_detail,
                'attach'      => $attach,
                'method'    => $method,
            ];
        } else {
            $params = $amount;
            $return_url = $params['return_url']?$params['return_url']:$config['return_url'];
            unset($params['return_url']);
        }
        $config = Service::getConfig();
        $request = request();
        $params['mchid'] = $config['mchid'];
        $key = $config['key'];
        $params['notify_url']?$params['notify_url']:$config['notify_url'];
        $return_url = $return_url?$return_url:$config['return_url']."/out_trade_no/" . $out_trade_no;
        $type = isset($params['type']) && in_array($params['type'], ['alipay', 'wechat']) ? $params['type'] : 'wechat';
        $method = isset($params['method']) ? $params['method'] : 'web';
        $isWechat = stripos($request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $response = [];
        if ($type == 'wechat') {
            if($method == 'web'){
                $method = $request->isMobile() ? 'wap' : $method;
                $method = $isWechat ? 'cashier' : $method;
            }
            //如果是PC支付,判断当前环境,进行跳转
            switch ($method) {
                case 'web':
                    //电脑支付,跳转到自定义展示页面(FastAdmin独有)
                    unset($params['method']);
                    $params['nonce_str'] = Service::getNonce();
                    $sign = Service::getPaySign($params,$key);
                    $params['sign'] = $sign;
                    $response = Service::curl("https://admin.xunhuweb.com/pay/payment",json_encode($params));
                    if($response['return_code'] == "SUCCESS"){
                        $response['body'] = $params['body'];
                        $response['return_url'] = $return_url;
                        Session::set("wechatresponse", $response);
                        $url = addon_url('xhpay/api/wechat', '', true, true);
                        return RedirectResponse::create($url);
                    }else{
                        echo $response['err_msg'];
                    }
                    break;
                case 'wap':
                    //手机网页支付,跳转
                    unset($params['method']);
                    $params['nonce_str'] = Service::getNonce();
                    $params['trade_type'] = "WAP";
                    $params['wap_url'] = $_SERVER['HTTP_REFERER'];
                    $params['wap_name'] = "fastAdmin";
                    $sign = Service::getPaySign($params,$key);
                    $params['sign'] = $sign;
                    $response = Service::curl("https://admin.xunhuweb.com/pay/payment",json_encode($params));
                    $url =$response['mweb_url'].'&redirect_url='.urlencode($return_url);
		            header("location: {$url}");
		            exit;
                    break;
                case 'cashier':
                    unset($params['method']);
                    unset($params['goods_detail']);
                    $params['nonce_str'] = Service::getNonce();
                    $params['redirect_url'] = $return_url;
                    $sign = Service::getPaySign($params,$key);
                    $params['sign'] = $sign;
                    $pay_url     = Service::data_link('https://admin.xunhuweb.com/pay/cashier', $params);
	        	    header("Location:". htmlspecialchars_decode($pay_url,ENT_NOQUOTES));
	        	    exit;
                    break;
                default:
            }
        } else {
            //如果是移动端自动切换为wap
            $method = $request->isMobile() ? 'cashier' : $method;
            switch ($method) {
                case 'web':
                    //电脑支付,跳转
                    unset($params['method']);
                    $params['nonce_str'] = Service::getNonce();
                    $sign = Service::getPaySign($params,$key);
                    $params['sign'] = $sign;
                    $response = Service::curl("https://admin.xunhuweb.com/pay/payment",json_encode($params));
                    if($response['return_code'] == "SUCCESS"){
                        $response['body'] = $params['body'];
                        $response['return_url'] = $return_url;
                        Session::set("alipayresponse", $response);
                        $url = addon_url('xhpay/api/alipay', '', true, true);
                        return RedirectResponse::create($url);
                    }else{
                        echo $response['err_msg'];
                    }
                    break;
                case 'cashier':
                    unset($params['method']);
                    unset($params['goods_detail']);
                    $params['nonce_str'] = Service::getNonce();
                    $params['redirect_url'] = $return_url;
                    $sign = Service::getPaySign($params,$key);
                    $params['sign'] = $sign;
                    $pay_url     = Service::data_link('https://admin.xunhuweb.com/alipaycashier', $params);
	        	    header("Location:". htmlspecialchars_decode($pay_url,ENT_NOQUOTES));
	        	    exit;
                    break;
                default:
            }
        }
    }
    
    public static function query($param)
    {
        $config = Service::getConfig();
        $param['mchid'] = $config['mchid'];
        $param['order_id'] = $param['order_id'];
        $param['nonce_str'] = Service::getNonce();
        $sign = Service::getPaySign($param,$config['key']);
        $param['sign'] = $sign;
        $response = Service::curl("https://admin.xunhuweb.com/pay/query",json_encode($param));
        return $response;
    }
    
    /**
     * 验证回调是否成功
     * @param string $type   支付类型
     * @param array  $config 配置信息
     * @return bool|Pay
     */
    public static function checkNotify()
    {
        $data=json_decode(file_get_contents('php://input'),true);
        $config = Service::getConfig();
        if(!$data){
            exit('faild!');
        }
        $hash =Service::getPaySign($data,$config['key']);
        if($data['sign']!=$hash){
            //签名验证失败
            return '签名验证失败';
        }
    	return $data;
    }

    /**
     * 获取配置
     * @param string $type 支付类型
     * @return array|mixed
     */
    public static function getConfig()
    {
        $config = get_addon_config('xhpay');
        return $config;
    }

    /**
     * curl 请求方法
     * $url 接口地址
     * $data 请求参数(一定要先转换为json，注意不要转码否则中文传过去被转码会审核失败等问题，json_ encode($param, 256)这样就可以)
     */
    public static function curl($url, $data){
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json; charset=utf-8',
                'Content-Length: ' . strlen($data)
            )
        );
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);    // 信任任何证
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);        // 表示不检查证书
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        return json_decode($response, true);
    }
    
    /**
     * url拼接
     * @param array $url
     * @param string $datas
     */
	public static function data_link($url,$datas){
		ksort($datas);
        reset($datas);
        $pre =array();
        foreach ($datas as $key => $data){
            if(is_null($data)||$data===''){
                continue;
            }
            $pre[$key]=$data;
        }

        $arg  = '';
        $qty = count($pre);
        $index=0;
		 foreach ($pre as $key=>$val){
		 		$val=urlencode($val);
			 	$arg.="$key=$val";
	            if($index++<($qty-1)){
	                $arg.="&amp;";
	            }	
        }
        return $url.'?'.$arg;
	}

    /**
     * 获取随机数
     */
    public static function getNonce(){
        static $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < 32; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    /**
     * 生成支付签名
     * @param array $data 参与签名的数据
     * @param string $hashkey 参与签名key
     * @return string
     */
    public static function getPaySign(array $datas, $hashkey)
    {
        ksort($datas);
        reset($datas);
        $pre =array();
        foreach ($datas as $key => $data){
            if(is_null($data)||$data===''){
                continue;
            }
            if($key=='method' || $key == 'sign'){
                continue;
            }
            $pre[$key]=$data;
        }

        $arg  = '';
        $qty = count($pre);
        $index=0;
        foreach ($pre as $key=>$val){
            $arg.="$key=$val";
            if($index++<($qty-1)){
                $arg.="&";
            }
        }
        return strtoupper(md5($arg.'&key='.$hashkey));
    }

}